#include <iostream>
#include <iomanip>
#include <math.h>
#include <chrono>

extern void initialize();
extern double f1();
extern double f2(double a);
extern double f3(double a);
extern double f4(double a);
extern double f5(double a);
extern double f6(double a, double b);
extern double f7(double a);

/*
 * In order to simplify the synchronisation in the parallel code, you can use
 * a holder class Double for the values, which implements the synchronisation,
 * like this:
 *
 * class Double {
 * private:
 * 	double val;
 * 	std::mutex mutex;
 * 	bool isValid;
 * 	std::condition_variable valid;
 * public:
 * 	Double() {
 * 		isValid = false;
 * 	}
 * 	void set(double v) {
 * 		// implement mutal exclusion!
 * 		val = v;
 *   	isValid = true;
 *      // notify all threads waiting for the value!
 * 	}
 * 	double get() {
 * 		// implement mutal exclusion!
 * 		// wait until the value is valid!
 * 		return val;
 * 	}
 * };
 *
 * You can then pass pointers to Double objects to the thread functions,
 * both for the input argument(s) and the result.
 */

int main(int argc, char **argv)
{
	initialize();
	
	auto start = std::chrono::high_resolution_clock::now();
	double a, b, c, res;
	a = f1();
    b = f2(a);
	c = f3(a);
	a = f4(a);
	res = f5(b) + f7(a) + f6(b,c);
	auto end = std::chrono::high_resolution_clock::now();
	
	std::cout << "Result = " << std::setprecision(18) << res << std::endl;
	std::cout << "Time: " << (end - start)/std::chrono::milliseconds(1) << "ms" << std::endl;
	return 0;
}
